=== modified file 'NEWS'
--- NEWS	2005-10-19 07:04:01 +0000
+++ NEWS	2005-10-20 02:17:07 +0000
@@ -32,6 +32,9 @@
 
     * New sftp transport, based on Paramiko.  (Robey Pointer)
 
+    * 'bzr pull' now accepts '--clobber' which will discard local changes
+      and make this branch identical to the source branch. (Robert Collins)
+
   TESTING:
 
     * The 'bzr selftest --pattern' option for has been removed, now 
@@ -45,6 +48,10 @@
     * 'bzr selftest' runs all tests, even if one feels, unless '--one'
       is given. (Martin Pool)
 
+    * There is a new method for TestCaseInTempDir, assertFileEqual, which
+      will check that a given content is equal to the content of the named
+      file. (Robert Collins)
+
   INTERNALS:
 
     * New 'testament' command and concept for making gpg-signatures 
@@ -97,6 +104,36 @@
     * Transports can register using register_lazy_transport, and they 
       will be loaded when first used.  (Martin Pool)
 
+    * 'pull' has been factored out of the command as WorkingTree.pull().
+      A new option to WorkingTree.pull has been added, clobber, which will
+      ignore diverged history and pull anyway.
+      (Robert Collins)
+
+    * config.Config has a 'get_user_option' call that accepts an option name.
+      This will be looked up in branches.conf and bazaar.conf as normal.
+      It is intended that this be used by plugins to support options - 
+      options of built in programs should have specific methods on the config.
+      (Robert Collins)
+
+    * merge.merge_inner now has tempdir as an optional parameter. (Robert
+      Collins)
+
+    * Tree.kind is not recorded at the top level of the hierarchy, as it was
+      missing on EmptyTree, leading to a bug with merge on EmptyTrees.
+      (Robert Collins)
+
+    * WorkingTree.__del__ has been removed, it was non deterministic and not 
+      doing what it was intended to. See WorkingTree.__init__ for a comment
+      about future directions. (Robert Collins/Martin Pool)
+
+    * bzrlib.transport.http has been modified so that only 404 urllib errors
+      are returned as NoSuchFile. Other exceptions will propogate as normal.
+      This allows debuging of actual errors. (Robert Collins)
+
+    * bzrlib.transport.Transport now accepts *ONLY* url escaped relative paths
+      to apis like 'put', 'get' and 'has'. This is to provide consistent
+      behaviour - it operates on url's only. (Robert Collins)
+
   BUG FIXES:
 
     * Better handling of branches in directories with non-ascii names. 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-18 06:30:47 +0000
+++ bzrlib/branch.py	2005-10-19 07:25:40 +0000
@@ -259,11 +259,11 @@
             self.text_store = get_store('text-store')
             self.revision_store = get_store('revision-store')
         elif self._branch_format == 5:
-            self.control_weaves = get_weave([])
+            self.control_weaves = get_weave('')
             self.weave_store = get_weave('weaves')
             self.revision_store = get_store('revision-store', compressed=False)
         elif self._branch_format == 6:
-            self.control_weaves = get_weave([])
+            self.control_weaves = get_weave('')
             self.weave_store = get_weave('weaves', prefixed=True)
             self.revision_store = get_store('revision-store', compressed=False,
                                             prefixed=True)
@@ -385,9 +385,11 @@
         return self._transport.abspath(name)
 
     def _rel_controlfilename(self, file_or_path):
-        if isinstance(file_or_path, basestring):
-            file_or_path = [file_or_path]
-        return [bzrlib.BZRDIR] + file_or_path
+        if not isinstance(file_or_path, basestring):
+            file_or_path = '/'.join(file_or_path)
+        if file_or_path == '':
+            return bzrlib.BZRDIR
+        return bzrlib.transport.urlescape(bzrlib.BZRDIR + '/' + file_or_path)
 
     def controlfilename(self, file_or_path):
         """Return location relative to branch."""
@@ -667,7 +669,7 @@
         []
         >>> WorkingTree(b.base, b).remove('foo')
         >>> list(b.unknowns())
-        ['foo']
+        [u'foo']
         """
         return self.working_tree().unknowns()
 
@@ -677,6 +679,10 @@
             mutter("add {%s} to revision-history" % revision_id)
         rev_history = self.revision_history()
         rev_history.extend(revision_ids)
+        self.set_revision_history(rev_history)
+
+    @needs_write_lock
+    def set_revision_history(self, rev_history):
         self.put_controlfile('revision-history', '\n'.join(rev_history))
 
     def has_revision(self, revision_id):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-19 05:03:12 +0000
+++ bzrlib/builtins.py	2005-10-19 11:54:59 +0000
@@ -334,10 +334,10 @@
     If branches have diverged, you can use 'bzr merge' to pull the text changes
     from one into the other.
     """
-    takes_options = ['remember']
+    takes_options = ['remember', 'clobber']
     takes_args = ['location?']
 
-    def run(self, location=None, remember=False):
+    def run(self, location=None, remember=False, clobber=False):
         from bzrlib.merge import merge
         import tempfile
         from shutil import rmtree
@@ -351,28 +351,12 @@
             else:
                 print "Using saved location: %s" % stored_loc
                 location = stored_loc
-        cache_root = tempfile.mkdtemp()
         br_from = Branch.open(location)
-        br_from.lock_read()
         try:
-            br_from.setup_caching(cache_root)
-            location = br_from.base
-            old_revno = br_to.revno()
-            old_revision_history = br_to.revision_history()
-            try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-            new_revision_history = br_to.revision_history()
-            if new_revision_history != old_revision_history:
-                merge(('.', -1), ('.', old_revno), check_clean=False)
-            if stored_loc is None or remember:
-                br_to.set_parent(location)
-        finally:
-            br_from.unlock()
-            rmtree(cache_root)
-
+            br_to.working_tree().pull(br_from, remember, clobber)
+        except DivergedBranches:
+            raise BzrCommandError("These branches have diverged."
+                                  "  Try merge.")
 
 
 class cmd_branch(Command):
@@ -439,7 +423,7 @@
                 copy_branch(br_from, to_location, revision_id, basis_branch)
             except bzrlib.errors.NoSuchRevision:
                 rmtree(to_location)
-                msg = "The branch %s has no revision %d." % (from_location, revision[0])
+                msg = "The branch %s has no revision %s." % (from_location, revision[0])
                 raise BzrCommandError(msg)
             except bzrlib.errors.UnlistableBranch:
                 msg = "The branch %s cannot be used as a --basis"

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2005-10-17 20:38:12 +0000
+++ bzrlib/config.py	2005-10-18 07:38:53 +0000
@@ -76,6 +76,14 @@
     def _get_signature_checking(self):
         """Template method to override signature checking policy."""
 
+    def _get_user_option(self, option_name):
+        """Template method to provide a user option."""
+        return None
+
+    def get_user_option(self, option_name):
+        """Get a generic option - no special process, no default."""
+        return self._get_user_option(option_name)
+
     def gpg_signing_command(self):
         """What program should be used to sign signatures?"""
         result = self._gpg_signing_command()
@@ -179,6 +187,13 @@
             if self._get_parser().has_option(section, 'email'):
                 return self._get_parser().get(section, 'email')
 
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        section = self._get_section()
+        if section is not None:
+            if self._get_parser().has_option(section, option_name):
+                return self._get_parser().get(section, option_name)
+
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
         section = self._get_section()
@@ -279,6 +294,14 @@
             return user_id
         return self._get_global_config()._get_user_id()
 
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        option_value = super(LocationConfig, 
+                             self)._get_user_option(option_name)
+        if option_value is not None:
+            return option_value
+        return self._get_global_config()._get_user_option(option_name)
+
     def _get_signature_checking(self):
         """See Config._get_signature_checking."""
         check = super(LocationConfig, self)._get_signature_checking()
@@ -315,6 +338,10 @@
         """See Config._get_signature_checking."""
         return self._get_location_config()._get_signature_checking()
 
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        return self._get_location_config()._get_user_option(option_name)
+
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
         return self._get_location_config()._gpg_signing_command()

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-17 23:13:00 +0000
+++ bzrlib/merge.py	2005-10-18 23:47:12 +0000
@@ -234,6 +234,7 @@
     eventually be done by just building the tree directly calling into 
     lower-level code (e.g. constructing a changeset).
     """
+    # RBC 20051019 is this not just 'export' ?
     merge((to_dir, -1), (to_dir, 0), this_dir=to_dir,
           check_clean=False, ignore_zero=True)
 
@@ -260,76 +261,72 @@
     All available ancestors of other_revision and base_revision are
     automatically pulled into the branch.
     """
-    tempdir = tempfile.mkdtemp(prefix="bzr-")
-    try:
-        if this_dir is None:
-            this_dir = '.'
-        this_branch = Branch.open_containing(this_dir)[0]
-        this_rev_id = this_branch.last_revision()
-        if this_rev_id is None:
-            raise BzrCommandError("This branch has no commits")
-        if check_clean:
-            changes = compare_trees(this_branch.working_tree(), 
-                                    this_branch.basis_tree(), False)
-            if changes.has_changed():
-                raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, this_branch)
-        if other_revision[1] == -1:
-            other_rev_id = other_branch.last_revision()
-            if other_rev_id is None:
-                raise NoCommits(other_branch)
-            other_basis = other_rev_id
-        elif other_revision[1] is not None:
-            other_rev_id = other_branch.get_rev_id(other_revision[1])
-            other_basis = other_rev_id
-        else:
-            other_rev_id = None
-            other_basis = other_branch.last_revision()
-            if other_basis is None:
-                raise NoCommits(other_branch)
-        if base_revision == [None, None]:
-            try:
-                base_rev_id = common_ancestor(this_rev_id, other_basis, 
-                                              this_branch)
-            except NoCommonAncestor:
-                raise UnrelatedBranches()
-            base_tree = get_revid_tree(this_branch, base_rev_id, None)
-            base_is_ancestor = True
-        else:
-            base_branch, base_tree = get_tree(base_revision)
-            if base_revision[1] == -1:
-                base_rev_id = base_branch.last_revision()
-            elif base_revision[1] is None:
-                base_rev_id = None
-            else:
-                base_rev_id = base_branch.get_rev_id(base_revision[1])
-            fetch(from_branch=base_branch, to_branch=this_branch)
-            base_is_ancestor = is_ancestor(this_rev_id, base_rev_id,
-                                           this_branch)
-        if file_list is None:
-            interesting_ids = None
-        else:
-            interesting_ids = set()
-            this_tree = this_branch.working_tree()
-            for fname in file_list:
-                path = this_tree.relpath(fname)
-                found_id = False
-                for tree in (this_tree, base_tree, other_tree):
-                    file_id = tree.inventory.path2id(path)
-                    if file_id is not None:
-                        interesting_ids.add(file_id)
-                        found_id = True
-                if not found_id:
-                    raise BzrCommandError("%s is not a source file in any"
-                                          " tree." % fname)
-        merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type, interesting_ids=interesting_ids)
-        if base_is_ancestor and other_rev_id is not None\
-            and other_rev_id not in this_branch.revision_history():
-            this_branch.add_pending_merge(other_rev_id)
-    finally:
-        shutil.rmtree(tempdir)
+    if this_dir is None:
+        this_dir = '.'
+    this_branch = Branch.open_containing(this_dir)[0]
+    this_rev_id = this_branch.last_revision()
+    if this_rev_id is None:
+        raise BzrCommandError("This branch has no commits")
+    if check_clean:
+        changes = compare_trees(this_branch.working_tree(), 
+                                this_branch.basis_tree(), False)
+        if changes.has_changed():
+            raise BzrCommandError("Working tree has uncommitted changes.")
+    other_branch, other_tree = get_tree(other_revision, this_branch)
+    if other_revision[1] == -1:
+        other_rev_id = other_branch.last_revision()
+        if other_rev_id is None:
+            raise NoCommits(other_branch)
+        other_basis = other_rev_id
+    elif other_revision[1] is not None:
+        other_rev_id = other_branch.get_rev_id(other_revision[1])
+        other_basis = other_rev_id
+    else:
+        other_rev_id = None
+        other_basis = other_branch.last_revision()
+        if other_basis is None:
+            raise NoCommits(other_branch)
+    if base_revision == [None, None]:
+        try:
+            base_rev_id = common_ancestor(this_rev_id, other_basis, 
+                                          this_branch)
+        except NoCommonAncestor:
+            raise UnrelatedBranches()
+        base_tree = get_revid_tree(this_branch, base_rev_id, None)
+        base_is_ancestor = True
+    else:
+        base_branch, base_tree = get_tree(base_revision)
+        if base_revision[1] == -1:
+            base_rev_id = base_branch.last_revision()
+        elif base_revision[1] is None:
+            base_rev_id = None
+        else:
+            base_rev_id = base_branch.get_rev_id(base_revision[1])
+        fetch(from_branch=base_branch, to_branch=this_branch)
+        base_is_ancestor = is_ancestor(this_rev_id, base_rev_id,
+                                       this_branch)
+    if file_list is None:
+        interesting_ids = None
+    else:
+        interesting_ids = set()
+        this_tree = this_branch.working_tree()
+        for fname in file_list:
+            path = this_tree.relpath(fname)
+            found_id = False
+            for tree in (this_tree, base_tree, other_tree):
+                file_id = tree.inventory.path2id(path)
+                if file_id is not None:
+                    interesting_ids.add(file_id)
+                    found_id = True
+            if not found_id:
+                raise BzrCommandError("%s is not a source file in any"
+                                      " tree." % fname)
+    merge_inner(this_branch, other_tree, base_tree, tempdir=None, 
+                ignore_zero=ignore_zero, backup_files=backup_files, 
+                merge_type=merge_type, interesting_ids=interesting_ids)
+    if base_is_ancestor and other_rev_id is not None\
+        and other_rev_id not in this_branch.revision_history():
+        this_branch.add_pending_merge(other_rev_id)
 
 
 def set_interesting(inventory_a, inventory_b, interesting_ids):
@@ -340,10 +337,30 @@
              source_file.interesting = source_file.id in interesting_ids
 
 
-def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
-                interesting_ids=None):
-
+def merge_inner(this_branch, other_tree, base_tree, tempdir=None, 
+                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
+                interesting_ids=None):
+    """Primary interface for merging. 
+
+    typical use is probably 
+    'merge_inner(branch, branch.get_revision_tree(other_revision),
+                 branch.get_revision_tree(base_revision))'
+    """
+    if tempdir is None:
+        _tempdir = tempfile.mkdtemp(prefix="bzr-")
+    else:
+        _tempdir = tempdir
+    try:
+        _merge_inner(this_branch, other_tree, base_tree, _tempdir,
+                     ignore_zero, merge_type, backup_files, interesting_ids)
+    finally:
+        if tempdir is None:
+            shutil.rmtree(_tempdir)
+
+
+def _merge_inner(this_branch, other_tree, base_tree, user_tempdir, 
+                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
+                interesting_ids=None):
     def merge_factory(file_id, base, other):
         contents_change = merge_type(file_id, base, other)
         if backup_files:

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2005-10-17 04:16:40 +0000
+++ bzrlib/option.py	2005-10-18 06:56:24 +0000
@@ -150,6 +150,7 @@
     Option.OPTIONS[name] = Option(name, **kwargs)
 
 _global_option('all')
+_global_option('clobber')
 _global_option('basis', type=str)
 _global_option('diff-options', type=str)
 _global_option('help',

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-18 06:30:47 +0000
+++ bzrlib/selftest/__init__.py	2005-10-18 13:11:57 +0000
@@ -212,7 +212,7 @@
         if not re.search(needle_re, haystack):
             raise AssertionError('pattern "%s" not found in "%s"'
                     % (needle_re, haystack))
-        
+
     def _enable_file_logging(self):
         fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
 
@@ -456,6 +456,11 @@
         """Fail unless path, which may be abs or relative, exists."""
         self.failUnless(osutils.lexists(path))
         
+    def assertFileEqual(self, content, path):
+        """Fail if path does not contain 'content'."""
+        self.failUnless(osutils.lexists(path))
+        self.assertEqualDiff(content, open(path, 'r').read())
+        
 
 class MetaTestLog(TestCase):
     def test_logging(self):

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-17 23:13:00 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-18 06:56:24 +0000
@@ -400,7 +400,15 @@
         self.runbzr('commit -m blah2 --unchanged')
         os.chdir('../b')
         self.runbzr('commit -m blah3 --unchanged')
+        # no clobber
         self.runbzr('pull ../a', retcode=1)
+        os.chdir('..')
+        self.runbzr('branch b clobberme')
+        os.chdir('clobberme')
+        self.runbzr('pull --clobber ../a')
+        clobbered = Branch.open('.')
+        self.assertEqual(clobbered.revision_history(),
+                         a.revision_history())
         os.chdir('../a')
         self.runbzr('merge ../b')
         self.runbzr('commit -m blah4 --unchanged')

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-17 23:13:00 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-19 07:25:40 +0000
@@ -62,19 +62,31 @@
         tree = b2.revision_tree('revision-1')
         eq(tree.get_file_text('foo-id'), 'hello')
 
-    def test_push_stores(self):
-        """Copy the stores from one branch to another"""
+    def get_unbalanced_branch_pair(self):
+        """Return two branches, a and b, with one file in a."""
         os.mkdir('a')
         br_a = Branch.initialize("a")
         file('a/b', 'wb').write('b')
         br_a.add('b')
-        commit(br_a, "silly commit")
-
+        commit(br_a, "silly commit", rev_id='A')
         os.mkdir('b')
         br_b = Branch.initialize("b")
+        return br_a, br_b
+
+    def get_balanced_branch_pair(self):
+        """Returns br_a, br_b as with one commit in a, and b has a's stores."""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        br_a.push_stores(br_b)
+        return br_a, br_b
+
+    def test_push_stores(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        # ensure the revision is missing.
         self.assertRaises(NoSuchRevision, br_b.get_revision, 
                           br_a.revision_history()[0])
         br_a.push_stores(br_b)
+        # check that b now has all the data from a's first commit.
         rev = br_b.get_revision(br_a.revision_history()[0])
         tree = br_b.revision_tree(br_a.revision_history()[0])
         for file_id in tree:
@@ -84,7 +96,7 @@
 
     def test_copy_branch(self):
         """Copy the stores from one branch to another"""
-        br_a, br_b = self.test_push_stores()
+        br_a, br_b = self.get_balanced_branch_pair()
         commit(br_b, "silly commit")
         os.mkdir('c')
         br_c = copy_branch(br_a, 'c', basis_branch=br_b)
@@ -104,7 +116,6 @@
         self.assertTrue(os.path.exists('b/one'))
         self.assertFalse(os.path.exists('b/two'))
         
-
     def test_record_initial_ghost_merge(self):
         """A pending merge with no revision present is still a merge."""
         branch = Branch.initialize('.')
@@ -158,6 +169,14 @@
                                         'FOO', 'A')
         self.assertEqual('FOO', branch.revision_store.get('A', 'sig').read())
 
+    def test__relcontrolfilename(self):
+        branch = Branch.initialize('.')
+        self.assertEqual('.bzr/%25', branch._rel_controlfilename('%'))
+        
+    def test__relcontrolfilename_empty(self):
+        branch = Branch.initialize('.')
+        self.assertEqual('.bzr', branch._rel_controlfilename(''))
+
 
 class TestRemote(TestCaseWithWebserver):
 

=== modified file 'bzrlib/selftest/testconfig.py'
--- bzrlib/selftest/testconfig.py	2005-10-17 10:50:39 +0000
+++ bzrlib/selftest/testconfig.py	2005-10-18 07:38:53 +0000
@@ -31,7 +31,8 @@
 sample_config_text = ("[DEFAULT]\n"
                       "email=Robert Collins <robertc@example.com>\n"
                       "editor=vim\n"
-                      "gpg_signing_command=gnome-gpg\n")
+                      "gpg_signing_command=gnome-gpg\n"
+                      "user_global_option=something\n")
 
 
 sample_always_signatures = ("[DEFAULT]\n"
@@ -58,6 +59,7 @@
                         "[/a/]\n"
                         "check_signatures=check-available\n"
                         "gpg_signing_command=false\n"
+                        "user_local_option=local\n"
                         "# test trailing / matching\n"
                         "[/a/*]\n"
                         "#subdirs will match but not the parent\n"
@@ -148,6 +150,10 @@
         my_config = config.Config()
         self.assertEqual('gpg', my_config.gpg_signing_command())
 
+    def test_get_user_option_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
 
 class TestConfigPath(TestCase):
 
@@ -268,19 +274,36 @@
                          my_config.signature_checking())
         self.assertEqual(False, my_config.signature_needed())
 
+    def _get_sample_config(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
     def test_gpg_signing_command(self):
-        config_file = StringIO(sample_config_text)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = self._get_sample_config()
         self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
         self.assertEqual(False, my_config.signature_needed())
 
+    def _get_empty_config(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
     def test_gpg_signing_command_unset(self):
-        config_file = StringIO("")
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = self._get_empty_config()
         self.assertEqual("gpg", my_config.gpg_signing_command())
 
+    def test_get_user_option_default(self):
+        my_config = self._get_empty_config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_get_user_option_global(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("something",
+                         my_config.get_user_option('user_global_option'))
+
 
 class TestLocationConfig(TestCase):
 
@@ -409,6 +432,16 @@
         self.get_location_config('/a')
         self.assertEqual("false", self.my_config.gpg_signing_command())
 
+    def test_get_user_option_global(self):
+        self.get_location_config('/a')
+        self.assertEqual('something',
+                         self.my_config.get_user_option('user_global_option'))
+
+    def test_get_user_option_local(self):
+        self.get_location_config('/a')
+        self.assertEqual('local',
+                         self.my_config.get_user_option('user_local_option'))
+
 
 class TestBranchConfigItems(TestCase):
 
@@ -454,3 +487,12 @@
         (my_config._get_location_config().
             _get_global_config()._get_parser(config_file))
         self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('something',
+                         my_config.get_user_option('user_global_option'))

=== modified file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	2005-10-17 02:16:12 +0000
+++ bzrlib/selftest/teststore.py	2005-10-19 07:25:40 +0000
@@ -353,3 +353,7 @@
         self.assertEqual('content', to_store.get('foo').read())
         self.assertEqual('signature', to_store.get('foo', 'sig').read())
         self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
+
+    def test_relpath_escaped(self):
+        my_store = store.TransportStore(MemoryTransport())
+        self.assertEqual('%25', my_store._relpath('%'))

=== modified file 'bzrlib/selftest/testtransport.py'
--- bzrlib/selftest/testtransport.py	2005-10-16 14:19:24 +0000
+++ bzrlib/selftest/testtransport.py	2005-10-19 07:25:40 +0000
@@ -21,7 +21,7 @@
 from bzrlib.errors import NoSuchFile, FileExists, TransportNotPossible
 from bzrlib.selftest import TestCase, TestCaseInTempDir
 from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
-from bzrlib.transport import memory
+from bzrlib.transport import memory, urlescape
 
 
 def _append(fn, txt):
@@ -32,6 +32,13 @@
     f.close()
     del f
 
+class TestTransport(TestCase):
+    """Test the non transport-concrete class functionality."""
+
+    def test_urlescape(self):
+        self.assertEqual('%25', urlescape('%'))
+
+
 class TestTransportMixIn(object):
     """Subclass this, and it will provide a series of tests for a Transport.
     It assumes that the Transport object is connected to the 
@@ -51,10 +58,11 @@
     def test_has(self):
         t = self.get_transport()
 
-        files = ['a', 'b', 'e', 'g']
+        files = ['a', 'b', 'e', 'g', '%']
         self.build_tree(files)
         self.assertEqual(t.has('a'), True)
         self.assertEqual(t.has('c'), False)
+        self.assertEqual(t.has(urlescape('%')), True)
         self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
                 [True, True, False, False, True, False, True, False])
         self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),

=== modified file 'bzrlib/selftest/testworkingtree.py'
--- bzrlib/selftest/testworkingtree.py	2005-10-17 23:32:06 +0000
+++ bzrlib/selftest/testworkingtree.py	2005-10-18 06:56:24 +0000
@@ -93,3 +93,26 @@
         self.assertEqual('w', tree.branch._lock_mode)
         tree.unlock()
         self.assertEqual(None, tree.branch._lock_count)
+ 
+    def get_pullable_branches(self):
+        self.build_tree(['from/', 'from/file', 'to/'])
+        br_a = Branch.initialize('from')
+        br_a.add('file')
+        br_a.commit('foo', rev_id='A')
+        br_b = Branch.initialize('to')
+        return br_a, br_b
+ 
+    def test_pull(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.working_tree().pull(br_a)
+        self.failUnless(br_b.has_revision('A'))
+        self.assertEqual(['A'], br_b.revision_history())
+
+    def test_pull_clobbers(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.commit('foo', rev_id='B')
+        self.assertEqual(['B'], br_b.revision_history())
+        br_b.working_tree().pull(br_a, clobber=True)
+        self.failUnless(br_b.has_revision('A'))
+        self.failUnless(br_b.has_revision('B'))
+        self.assertEqual(['A'], br_b.revision_history())

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-10-17 02:16:12 +0000
+++ bzrlib/store/__init__.py	2005-10-19 07:25:40 +0000
@@ -234,7 +234,7 @@
         else:
             path = [fileid]
         path.extend(suffixes)
-        return '.'.join(path)
+        return transport.urlescape('.'.join(path))
 
     def __repr__(self):
         if self._transport is None:

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2005-10-16 14:51:21 +0000
+++ bzrlib/store/weave.py	2005-10-20 01:05:07 +0000
@@ -20,9 +20,10 @@
 # files whose id differs only in case.  That should probably be forbidden.
 
 
+import errno
+import os
 from cStringIO import StringIO
-import os
-import errno
+import urllib
 
 from bzrlib.weavefile import read_weave, write_weave_v5
 from bzrlib.weave import Weave
@@ -47,9 +48,9 @@
     def filename(self, file_id):
         """Return the path relative to the transport root."""
         if self._prefixed:
-            return hash_prefix(file_id) + file_id + WeaveStore.FILE_SUFFIX
+            return hash_prefix(file_id) + urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
         else:
-            return file_id + WeaveStore.FILE_SUFFIX
+            return urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
 
     def __iter__(self):
         l = len(WeaveStore.FILE_SUFFIX)

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-10-19 06:44:53 +0000
+++ bzrlib/transport/__init__.py	2005-10-20 02:17:07 +0000
@@ -366,8 +366,16 @@
         klass = getattr(mod, classname)
         return klass(base)
     register_transport(scheme, _loader)
-    
-# If nothing else matches, try the LocalTransport
+
+
+def urlescape(relpath):
+    """Escape relpath to be a valid url."""
+    # TODO utf8 it first. utf8relpath = relpath.encode('utf8')
+    import urllib
+    return urllib.quote(relpath)
+
+
+# None is the default transport, for things with no url scheme
 register_lazy_transport(None, 'bzrlib.transport.local', 'LocalTransport')
 register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
 register_lazy_transport('sftp://', 'bzrlib.transport.sftp', 'SFTPTransport')

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-10-19 06:44:53 +0000
+++ bzrlib/transport/http.py	2005-10-20 02:17:07 +0000
@@ -74,6 +74,7 @@
         """Return the full url to the given relative path.
         This can be supplied with a string or a list
         """
+        assert isinstance(relpath, basestring)
         if isinstance(relpath, basestring):
             relpath_parts = relpath.split('/')
         else:
@@ -125,10 +126,10 @@
             f.read()
             f.close()
             return True
-        except BzrError:
-            return False
-        except urllib2.URLError:
-            return False
+        except urllib2.URLError, e:
+            if e.code == 404:
+                return False
+            raise
         except IOError, e:
             if e.errno == errno.ENOENT:
                 return False
@@ -141,7 +142,13 @@
         """
         try:
             return get_url(self.abspath(relpath))
-        except (BzrError, urllib2.URLError, IOError), e:
+        except urllib2.URLError, e:
+            if e.code == 404:
+                raise NoSuchFile(msg = "Error retrieving %s: %s" 
+                                 % (self.abspath(relpath), str(e)),
+                                 orig_error=e)
+            raise
+        except (BzrError, IOError), e:
             raise NoSuchFile(msg = "Error retrieving %s: %s" 
                              % (self.abspath(relpath), str(e)),
                              orig_error=e)

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-19 06:44:53 +0000
+++ bzrlib/transport/local.py	2005-10-20 02:17:07 +0000
@@ -23,6 +23,7 @@
 import shutil
 from stat import ST_MODE, S_ISDIR, ST_SIZE
 import tempfile
+import urllib
 
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport, register_transport, \
@@ -59,12 +60,11 @@
             return LocalTransport(self.abspath(offset))
 
     def abspath(self, relpath):
-        """Return the full url to the given relative path.
+        """Return the full url to the given relative URL.
         This can be supplied with a string or a list
         """
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
-        return os.path.join(self.base, *relpath)
+        assert isinstance(relpath, basestring)
+        return os.path.join(self.base, urllib.unquote(relpath))
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-10-19 06:44:53 +0000
+++ bzrlib/transport/sftp.py	2005-10-20 02:17:07 +0000
@@ -21,6 +21,7 @@
 import re
 import stat
 import sys
+import urllib
 
 from bzrlib.errors import TransportNotPossible, NoSuchFile, NonRelativePath, TransportError
 from bzrlib.config import config_dir
@@ -125,8 +126,8 @@
     def _abspath(self, relpath):
         """Return the absolute path segment without the SFTP URL."""
         # FIXME: share the common code across transports
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
+        assert isinstance(relpath, basestring)
+        relpath = [urllib.unquote(relpath)]
         basepath = self._path.split('/')
         if len(basepath) > 0 and basepath[-1] == '':
             basepath = basepath[:-1]

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-10-14 04:32:24 +0000
+++ bzrlib/tree.py	2005-10-18 23:47:12 +0000
@@ -71,6 +71,9 @@
     def id2path(self, file_id):
         return self.inventory.id2path(file_id)
 
+    def kind(self, file_id):
+        raise NotImplementedError("subclasses must implement kind")
+
     def _get_inventory(self):
         return self._inventory
     
@@ -182,6 +185,7 @@
     def kind(self, file_id):
         return self._inventory[file_id].kind
 
+
 class EmptyTree(Tree):
     def __init__(self):
         self._inventory = Inventory()
@@ -192,6 +196,10 @@
     def has_filename(self, filename):
         return False
 
+    def kind(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return "root_directory"
+
     def list_files(self):
         return iter([])
     

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-18 06:30:47 +0000
+++ bzrlib/workingtree.py	2005-10-19 06:01:01 +0000
@@ -49,9 +49,10 @@
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock, quotefn
 import bzrlib.tree
 from bzrlib.osutils import appendpath, file_kind, isdir, splitpath, relpath
-from bzrlib.errors import BzrCheckError, NotVersionedError
+from bzrlib.errors import BzrCheckError, DivergedBranches, NotVersionedError
 from bzrlib.trace import mutter
 
+
 class TreeEntry(object):
     """An entry that implements the minium interface used by commands.
 
@@ -140,6 +141,10 @@
 
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
         hc = self._hashcache = HashCache(basedir)
         hc.read()
         hc.scan()
@@ -147,12 +152,6 @@
         if hc.needs_write:
             mutter("write hc")
             hc.write()
-            
-            
-    def __del__(self):
-        if self._hashcache.needs_write:
-            self._hashcache.write()
-
 
     def __iter__(self):
         """Iterate through file_ids for this tree.
@@ -330,6 +329,32 @@
                 conflicted.add(stem)
                 yield stem
 
+    @needs_write_lock
+    def pull(self, source, remember=False, clobber=False):
+        from bzrlib.merge import merge_inner
+        source.lock_read()
+        try:
+            old_revision_history = self.branch.revision_history()
+            try:
+                self.branch.update_revisions(source)
+            except DivergedBranches:
+                if not clobber:
+                    raise
+                self.branch.set_revision_history(source.revision_history())
+            new_revision_history = self.branch.revision_history()
+            if new_revision_history != old_revision_history:
+                if len(old_revision_history):
+                    other_revision = old_revision_history[-1]
+                else:
+                    other_revision = None
+                merge_inner(self.branch,
+                            self.branch.basis_tree(), 
+                            self.branch.revision_tree(other_revision))
+            if self.branch.get_parent() is None or remember:
+                self.branch.set_parent(source.base)
+        finally:
+            source.unlock()
+
     def extras(self):
         """Yield all unknown files in this WorkingTree.
 

